import io from "socket.io-client";
import React, {useEffect, useState} from "react";
import {useRouter} from "next/router";
import Layout from "../../components/Layout";
import {DeleteIcon, LockIcon, ManageIcon, OptionsIcon, ShareIcon, PublicIcon} from "../../components/icons";

let socket;

export default function Document() {
    const router = useRouter();

    const [text, setText] = useState("");
    const [token, setToken] = useState("");
    const [title, setTitle] = useState("");
    const [shared, setShared] = useState(false);
    const [publicMode, setPublicMode] = useState(false);

    useEffect(() => {
        socketInitializer();
    }, [router.query.id]);

    useEffect(() => {
        setToken(() => localStorage.getItem("token"))
    }, [])

    useEffect(() => {
        if (token && router.query.id) {
            const fetchData = async () => {
                const res = await fetch(`http://localhost:8000/api/documents/${router.query.id}`, {
                    method: "GET",
                    headers: {
                        'Authorization': `Bearer ${token}`
                    }
                })
                const json = await res.json();
                setText(() => json["data"])
                setTitle(() => json["name"])
                setShared(() => json["owner"].toString() !== localStorage.getItem("user"))
                setPublicMode(() => json["public"])
            }

            fetchData()
        }
    }, [token, router.query.id])

    useEffect(() => {
        if (token && router.query.id) {
            const interval = setInterval(() => {
                fetch(`http://localhost:8000/api/documents/${router.query.id}`, {
                    method: 'PUT',
                    body: `{"data": "${text.replace(/\n/g, "\\n")}"}`,
                    headers: {
                        'Content-type': 'application/json',
                        'Authorization': `Bearer ${token}`
                    }
                })
            }, 500);

            return () => clearInterval(interval);
        }
    }, [token, text, router.query.id])

    const socketInitializer = async () => {
        await fetch("/api/socket");

        socket = io();

        socket.emit('create', router.query.id);

        socket.on("newIncomingText", (text) => {
            setText(() => text);
            console.log('text' + text);
        });
    };

    const sendText = async () => {
        socket.emit("createdText", text, router.query.id);
        setText(() => text);
    };

    const handleKey = () => {
        if (text) {
            sendText();
        }
    };

    let access = router.query.id + "/access"
    const deleteDoc = async () => {
        const res = await fetch(`http://localhost:8000/api/documents/${router.query.id}`, {
            method: "DELETE",
            headers: {
                'Authorization': `Bearer ${token}`
            }
        })
        const json = await res.json();

        if (res.status == 201) {
            router.push("/")
        }
    };
    const publicOrPrivate = async () => {
        if (!publicMode) {
            const res = await fetch(`http://localhost:8000/api/documents/${router.query.id}`, {
                method: "PUT",
                headers: {
                    'Content-type': 'application/json',
                    'Authorization': `Bearer ${token}`
                },
                body: '{"public": true}'
            })

            if (res.status == 202) {
                setPublicMode(() => true)
            }
        } else {
            const res = await fetch(`http://localhost:8000/api/documents/${router.query.id}`, {
                method: 'PUT',
                headers: {
                    'Content-type': 'application/json',
                    'Authorization': `Bearer ${token}`
                },
                body: '{"public": false}'
            })

            if (res.status == 202) {
                setPublicMode(() => false)
            }
        }
    };

    return (
        <Layout>
            <div style={{height: "50px", justifyContent: "space-between"}}
                 className="align-center flex">
              <span className="docTitle">
                  {title}
              </span>
                <div className="dropdown">
                    {!shared && (<>
                        <OptionsIcon onClick={(e) => {
                            document.getElementById("myDropdown").classList.toggle("show");

                            window.onclick = function (event) {
                                if (!event.target.matches('.dropbtn')) {
                                    var dropdowns = document.getElementsByClassName("dropdown-content");
                                    var i;
                                    for (i = 0; i < dropdowns.length; i++) {
                                        var openDropdown = dropdowns[i];
                                        if (openDropdown.classList.contains('show')) {
                                            openDropdown.classList.remove('show');
                                        }
                                    }
                                }
                            }
                        }} className="dropbtn"></OptionsIcon>
                        <div id="myDropdown" className="dropdown-content">
                            <a href={access}>
                                <div className="flex flex-row items-center justify-start align-center menu-opt">
                                    <ManageIcon/><span>Manage access</span>
                                </div>
                            </a>
                            {publicMode && <a onClick={publicOrPrivate}>
                                <div className="flex flex-row items-center justify-start align-center menu-opt">
                                    <LockIcon/><span>Make private</span></div>
                            </a>
                            }
                            {!publicMode && <a onClick={publicOrPrivate}>
                                <div className="flex flex-row items-center justify-start align-center menu-opt">
                                    <PublicIcon/><span>Make public</span></div>
                            </a>
                            }
                            <a onClick={deleteDoc}>
                                <div className="flex flex-row items-center justify-start align-center menu-opt">
                                    <DeleteIcon/><span>Delete document</span></div>
                            </a>
                        </div>
                    </>)}
                </div>
            </div>
            <textarea
                className="document_area"
                placeholder="Type..."
                onKeyUp={handleKey}
                value={text}
                onChange={(e) => setText(e.target.value)}
            />
        </Layout>
    );
}