import React, {useEffect, useState} from "react";
import {useRouter} from "next/router";
import Layout from "../../../components/Layout";
import {DeleteIcon} from "../../../components/icons";

export default function Document() {
    const router = useRouter();

    const [token, setToken] = useState("");
    const [users, setUsers] = useState([]);
    const [update, setUpdate] = useState(false);

    useEffect(() => {
        setToken(() => localStorage.getItem("token"))
    }, [])

    useEffect(() => {
        if (token && router.query.id) {
            const fetchData = async () => {
                const res = await fetch(`http://localhost:8000/api/documents/${router.query.id}/access`, {
                    method: "GET",
                    headers: {
                        'Authorization': `Bearer ${token}`
                    }
                })
                const json = await res.json();
                let list = []
                json.forEach((user) => {
                    list.push({
                        id: user[0],
                        username: user[1],
                        access: user[2]
                    })
                })
                setUsers(() => list)
            }

            fetchData()
        }
    }, [token, router.query.id, update])

    const handleSubmitAccess = async (event, username) => {
        event.preventDefault()

        console.log(event.target.value)
        console.log(event.target)

        let data = {
            username: username,
            access_type: event.target.value,
        }

        const JSONdata = JSON.stringify(data)

        console.log(JSONdata)

        const res = await fetch(`http://localhost:8000/api/documents/${router.query.id}/access`, {
            method: "POST",
            headers: {
                'Content-Type': 'application/json',
                'Authorization': `Bearer ${token}`
            },
            body: JSONdata
        })
        let json = await res.json();
        // setUsers(() => [])
        setUpdate(() => update == true ? false : true)
    }

    const handleSubmitDelete = async (event, username) => {
        event.preventDefault()

        let data = {
            username: username,
        }

        const JSONdata = JSON.stringify(data)

        const res = await fetch(`http://localhost:8000/api/documents/${router.query.id}/access`, {
            method: "DELETE",
            headers: {
                'Content-Type': 'application/json',
                'Authorization': `Bearer ${token}`
            },
            body: JSONdata
        })
        let json = await res.json();
        setUsers(() => [])
        setUpdate(() => update == true ? false : true)
    }

    const handleSubmit = async (event) => {
        event.preventDefault()

        const aux = await fetch(`http://localhost:8000/api/users?` + new URLSearchParams(
            {user: event.target.name.value}
        ), {
            method: "GET",
            headers: {
                'Authorization': `Bearer ${token}`
            }
        })
        const pr = await aux.json();
        let user_id = pr[0]

        let data = {
            username: user_id,
            access_type: event.target.select.value,
        }

        const JSONdata = JSON.stringify(data)

        const res = await fetch(`http://localhost:8000/api/documents/${router.query.id}/access`, {
            method: "POST",
            headers: {
                'Content-Type': 'application/json',
                'Authorization': `Bearer ${token}`
            },
            body: JSONdata
        })
        let json = await res.json();
        setUpdate(() => update == true ? false : true)
    }


    return (
        <Layout>
            <div style={{justifyContent: "space-between"}}
                 className="align-center flex">
                <table>
                    <thead>
                    <tr>
                        <th>Username</th>
                        <th>Access level</th>
                        <th>Remove</th>
                    </tr>
                    </thead>
                    <tbody>
                    {users.map(({icon: Icon, ...user}) => {
                        return (
                            <tr style={{color: "black"}} key={user.id}>
                                <td>{user.username}</td>
                                <td>
                                    <select style={{padding: "4px"}} value={user.access}
                                            onChange={(e) => handleSubmitAccess(e, user.id)}>
                                        <option value={1} label="Full-access"></option>
                                        <option value={2} label="Edit"></option>
                                        <option value={3} label="Read-only"></option>
                                    </select>

                                </td>
                                <td>
                                    <DeleteIcon onClick={(e) => handleSubmitDelete(e, user.id)}/>
                                </td>
                            </tr>
                        );
                    })}
                    </tbody>
                </table>

                <div className="flex items-center justify-center w-full h-full">
                    <form onSubmit={handleSubmit}
                          className="flex items-center justify-center align-center w-full h-full login-form"
                          style={{flexDirection: "column"}}>
                        <div>
                            <input type="text" id="name" name="name" required placeholder="name"
                                   style={{border: "1px solid grey", color: "black", padding: "2px 4px"}}/>
                        </div>
                        <div>
                            <select id="select" style={{padding: "4px"}}>
                                <option value={1} label="Full-access"></option>
                                <option value={2} label="Edit"></option>
                                <option value={3} label="Read-only"></option>
                            </select>
                        </div>

                        <button type="submit" className="rounded bg-pink-600 rounded"
                                style={{border: "1px solid grey", padding: "4px"}}>Add user
                        </button>
                    </form>
                </div>
                {/*</>*/}
                {/*    );*/}
                {/*})}*/}
            </div>
        </Layout>
    );
}